/* ---------------------------------------------------- *
 * dl_test.c - simple code to test libhavege.so (dlopen)*
 * ---------------------------------------------------- *
 * Copyright (c) 2006 - Olivier Rochecouste             *
 *                                                      *
 * This program is free software; you can redistribute  *
 * it and/or modify it under the terms of the GNU       *
 * General Public License as published by the Free      *
 * Software Foundation; either version 2 of the License,*
 * or (at your option) any later version.               *
 *                                                      *
 * This program is distributed in the hope that it will *
 * be useful, but WITHOUT ANY WARRANTY; without even the*
 * implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE. See the GNU General Public       *
 * License for more details.                            *
 *                                                      *
 * You should have received a copy of the GNU General   *
 * Public License along with this program; if not, write*
 * to the Free Software Foundation, Inc., 675 Mass Ave, *
 * Cambridge, MA 02139, USA.                            *
 *                                                      *
 * contact information: orocheco [at] irisa [dot] fr    *
 * ==================================================== */

#include <stdlib.h>
#include <stdio.h>

/* dlopen API */

#include <dlfcn.h>

/* ------------------ *
 * shared object NAME *
 * ------------------ */

#define SONAME "libhavege.so"
#define NDRAND "ndrand"
#define CRRAND "crypto_ndrand"

#define _32MB 32000*1024
#define _08MB  8000*1024
#define _04MB  4000*1024
#define _02MB  2000*1024

static int random_pool1 [_32MB];
static int random_pool2 [_08MB];

/* --------------------------------------------------- *
 * dump the content of the random pool in file "fname" *
 * --------------------------------------------------- */

#ifdef DUMP_CONTENT
void
dump_bin (char *fname, int *buf, int size)
{
  FILE *fout;

  if (!(fout = fopen (fname, "wb"))) {
    fprintf(stderr, "cannot open file <%s> for writing.\n",
	    fname);
    return;
  }

  if (fwrite (buf, sizeof (int), size, fout) == 0) {
    fprintf(stderr, "cannot write data in file: %s.\n",
	    fname);
    return;
  }

  fclose (fout);
}
#endif

int
main(int argc, char **argv)
{
  int i; char *error;

#ifdef DUMP_CONTENT
  char *fname1 = "ndrand.out";
  char *fname2 = "crrand.out";
#endif

  /* HAVEGE RNG methods */

  int (*ndrand)(void);
  int (*crypto_ndrand)(void);

  void *handle;

  /* -------------------------- *
   * 1. load the shared library *
   * -------------------------- */

  handle = dlopen (SONAME, RTLD_LAZY);
  if (!handle) {
    fputs (dlerror(), stderr);
    exit(1);
  }

  /* --------------- *
   * 2. find symbols *
   * --------------- */

  ndrand = dlsym(handle, NDRAND);
  if ((error = dlerror()) != NULL)  {
    fputs(error, stderr);
    exit(1);
  }

  crypto_ndrand = dlsym(handle, CRRAND);
  if ((error = dlerror()) != NULL)  {
    fputs(error, stderr);
    exit(1);
  }

  /* -------------------------- */
  /* 3. generate random numbers */
  /* -------------------------- */

  /* using ndrand() */

  for (i = 0; i < _32MB/sizeof(int); i++) {
    random_pool1[i] = ndrand ();
  }

#ifdef DUMP_CONTENT
  dump_bin (fname1, random_pool1, _32MB/sizeof(int));
#endif

#ifdef RUN_NIST
  /* perform NIST tests on the first _04MB of data */
  if (PackTestF (random_pool1, _04MB, "ndrand_nist1.out") < 8) {
    PackTestL (random_pool1, _04MB, "ndrand_nist1.out");
  }
#endif

  /* using crypto_ndrand() */

  for (i = 0; i < _08MB/sizeof(int); i++) {
    random_pool2[i] = crypto_ndrand ();
  }

#ifdef DUMP_CONTENT
  dump_bin (fname2, random_pool2, _08MB/sizeof(int));
#endif

#ifdef RUN_NIST
  /* perform NIST tests on the first _04MB of data */
  if (PackTestF (random_pool2, _02MB, "crrand_nist2.out") < 8) {
    PackTestL (random_pool2, _02MB, "crrand_nist2.out");
  }
#endif

  /* ---------------------------- */
  /* 4. unload the shared library */
  /* ---------------------------- */

  dlclose(handle);
}
